@echo off
REM Licensed Materials - Property of IBM
REM
REM (C) Copyright IBM Corp. 2013 All Rights Reserved
REM
REM US Government Users Restricted Rights - Use, duplicate or
REM disclosure restricted by GSA ADP Schedule Contract with
REM IBM Corp.
REM IBM_COPYRIGHT_END

setlocal

set RETURN_CODE=0
set logFile=""
if not '%2'=='' goto usage
for /f "skip=1 delims=." %%c in ('wmic os get LocalDateTime') do (
	set timestamp=%%c
	goto updateJRE
)
goto updateJRE

:usage
	echo Usage:
	echo updateServerJRE_always.bat [root path of Director]
	exit /B 0

:updateJRE
	rem * get default path from dir like C:\Program Files\IBM\Director\lwi\conf\exescripts\prelaunch_always\
	set "scriptDir=%~dp0"
	set "rootPath=%scriptDir:~0,-38%"
	if not '%1'=='' (
		set "rootPath=%~1"
	)
	if not exist "%rootPath%" (
		echo The root path of Director does not exist: "%rootPath%"
		set RETURN_CODE=1
		goto endlocal
	)
	
	set "logdir="%rootPath%\log""
	if not exist %logdir% (
		mkdir %logdir%
		echo Created log folder: %logdir%
	)
	set logFile="%rootPath%\log\updateinstaller.log"
	set "deleteUpdateJREFile=%rootPath%\lwi\conf\exescripts\prelaunch_once\cleanUpdateJREScript.bat"
	set "jrePath=%rootPath%\jre"
	set "tmpJre=%rootPath%\jre.new"
	set "stamp=%timestamp%"
	REM set "oldJre=%rootPath%\jre.old.%stamp%"  Commented it because the timestamep is different after next restart.
	set "oldJre=%rootPath%\jre.old"
	set "javaHomeFile=%rootPath%\lwi\conf\javaHome.bat%"
	set "javaHomeFileOld=%rootPath%\lwi\conf\javaHome.bat.old.need%"
	set "javaHome=%jrePath%"
	
	echo The log file is %logFile%                                             >> %logFile%
	if not exist "%javaHomeFileOld%" (
		echo Trying to update JRE by prelaunch_always at %date% %time%         >> %logFile%
	) else (
		echo Trying to switch JRE dir at %date% %time%  	                   >> %logFile%
		echo This is Not First Restart after JRE feature update.               >> %logFile%
	)
	
	if not exist "%tmpJre%" (
		echo The new JRE is not ready in "%tmpJre%"                            >> %logFile%
		goto endlocal
	) else (
		set "javaHome=%tmpJre%"
	)
	
	call :killOtherProcesses
	
	if not exist "%jrePath%" (
		echo Trying to use jre dir...                                          >> %logFile%
		call :moveNewJre
		call :writeJavaHomeToFile
		call :echoJaveVersion
		goto endlocal
	)
	
	move /Y "%jrePath%" "%oldJre%"                                             >> %logFile% 2>&1
	set RETURN_CODE=%ERRORLEVEL%
	if "%RETURN_CODE%" == "0" (
		echo Moved "%jrePath%" to "%oldJre%"                                   >> %logFile%
		call :moveNewJre
	) else (
		echo Failed to move "%jrePath%" to "%oldJre%"                          >> %logFile%
		echo The JRE dir "%jrePath%" may be locked.                            >> %logFile%
		if not exist "%javaHomeFileOld%" (
			call :resetJavaHome
		) else (
			echo Attempting removal of old JRE    	                           >> %logFile%
			rmdir /S /Q "%jrePath%"                                            >> %logFile% 2>&1
			if not exist "%jrePath%" (
				echo Successfully removed old JRE "%jrePath%"                  >> %logFile%
			) else (
				echo Removal of old JRE "%jrePath%" failed, files or directories may remain.       >> %logFile%
			)
		)
		echo Prelaunch will try to switch jre dir at next restart.             >> %logFile%
	)
	
	call :writeJavaHomeToFile
	call :echoJaveVersion
	goto endlocal

:moveNewJre
	move /Y "%tmpJre%" "%jrePath%"                                             >> %logFile%
	set RETURN_CODE=%ERRORLEVEL%
	if "%RETURN_CODE%" == "0" (
		set "javaHome=%jrePath%"
		echo Moved new JRE from "%tmpJre%" to "%jrePath%" successfully.        >> %logFile%
		echo Completed jre dir switch.                                         >> %logFile%
		if exist "%javaHomeFileOld%" (
			del /F /Q "%javaHomeFileOld%"                                      >> %logFile%
		)
		if exist "%javaHomeFileOld%" (
			echo Failed to delete "%javaHomeFileOld%"                          >> %logFile%
		)
	) else (
		set "javaHome=%tmpJre%"
		echo Failed to move new JRE from "%tmpJre%" to "%jrePath%"             >> %logFile%
		if not exist "%javaHomeFileOld%" (
			call :resetJavaHome
		)
	)
	if exist "%oldJre%" (
		rmdir /S /Q "%oldJre%"                                                 >> %logFile% 2>&1
		if exist "%oldJre%" (
			echo Failed to delete "%oldJre%" at this time.                     >> %logFile%
		) else (
			echo Deleted "%oldJre%" successfully.                              >> %logFile%
		)
	)
	goto :EOF
	
	
:resetJavaHome
	set "javaHome=%tmpJre%"
	
	echo Trying to reset JAVA HOME...                                      >> %logFile%
	move /Y "%javaHomeFile%" "%javaHomeFileOld%"                           >> %logFile%
	if not exist "%javaHomeFileOld%" (
		echo Failed to move "%javaHomeFile%" to "%javaHomeFileOld%"        >> %logFile%
	) else (
		echo Moved "%javaHomeFile%" to "%javaHomeFileOld%"                 >> %logFile%
	)
	
	if exist "%jrePath%" (
		echo Deleting "%jrePath%" ...                                          >> %logFile%
		rmdir /S /Q "%jrePath%"                                                >> %logFile% 2>&1
		echo Executed rmdir "%jrePath%" to delete current jre dir and files.   >> %logFile%
	)
	goto :EOF
	
:writeJavaHomeToFile
	del /F /Q "%javaHomeFile%"                                                 >> %logFile%
	if exist "%javaHomeFile%" (
		echo ERROR: Failed to delete "%javaHomeFile%" to change Java Home.     >> %logFile%
	)
	echo set JAVA_HOME=%javaHome%> "%javaHomeFile%"
	if not exist "%javaHomeFile%" (
		echo ERROR: Failed to create a new "%javaHomeFile%" to change Java Home. >> %logFile%
	)
	goto :EOF
	
:echoJaveVersion
	echo Current Java HOME in javaHome.bat is:                                 >> %logFile%
	type "%javaHomeFile%"                                                      >> %logFile%
	echo javaHome variable is: %javaHome%                                      >> %logFile%
	"%javaHome%\bin\java" -version                                             2>> %logFile%
	goto :EOF
	
:killOtherProcesses
	echo Finding all processes running director JRE...                         >> %logFile%
	wmic process where "CommandLine like '%%Director\\jre\\bin\\%%'"           >> %logFile% 2>&1
	echo ------                                                                >> %logFile%
	echo Killing processes running director JRE...                             >> %logFile%
	wmic process where "CommandLine like '%%Director\\jre\\bin\\%%'" delete    >> %logFile%
	echo Killing processes running director tmp JRE (jre.new)...               >> %logFile%
	wmic process where "CommandLine like '%%Director\\jre.new\\bin\\%%'" delete                    >> %logFile%
	echo Killing processes locking javaHome.bat...                             >> %logFile%
	wmic process where "CommandLine like '%%Director\\lwi\\conf\\javaHome.bat%%'" delete           >> %logFile%
	echo Killing processes locking javaHome.bat.old.need...                    >> %logFile%
	wmic process where "CommandLine like '%%Director\\lwi\\conf\\javaHome.bat.old.need%%'" delete  >> %logFile%
	goto :EOF

:endlocal
	if not %logFile% == "" (
		if not exist "%tmpJre%" (
			REM The script can delte itself, but it may affect exit /B 0, then LWI will be stuck.
			REM del /F /Q "%~f0"                                               >> %logFile%
			REM Create a file into prelaunch_once to delete the script.
			echo ^@echo off> "%deleteUpdateJREFile%"
			echo del /F /Q "%~f0" >> "%deleteUpdateJREFile%"
			echo echo The JRE update cleanup is done at %%date%% %%time%% ^>^> %logFile% >> "%deleteUpdateJREFile%"
			echo exit /B 0 >> "%deleteUpdateJREFile%"
			if not exist "%deleteUpdateJREFile%" (
				echo The file "%deleteUpdateJREFile%" is not created. you may manually remove "%~f0"              >> %logFile%
			) else (
				echo Created the file "%deleteUpdateJREFile%" to clean the update script "%~f0" at next restart.  >> %logFile%
			)
		)
		echo The update ended at %date% %time%                                 >> %logFile%
		echo ----------------------------------------------------------------  >> %logFile%
	)
	endlocal
	
	REM Apply new JAVA_HOME for launchlwi.bat
	call "%CONF_DIR%\javaHome.bat"
	set JAVA_PROGRAM=%JAVA_HOME%\bin\%JAVA_EXECUTABLE%
	exit /B 0