@echo off
REM Licensed Materials - Property of IBM
REM
REM (C) Copyright IBM Corp. 2013 All Rights Reserved
REM
REM US Government Users Restricted Rights - Use, duplicate or
REM disclosure restricted by GSA ADP Schedule Contract with
REM IBM Corp.
REM IBM_COPYRIGHT_END

setlocal

set RETURN_CODE=0
set logFile=""
if not '%2'=='' goto usage
for /f "skip=1 delims=." %%c in ('wmic os get LocalDateTime') do (
	set timestamp=%%c
	goto updateJRE
)
goto updateJRE

:usage
	echo Usage:
	echo updateServerJRE.bat [root path of Director]
	exit /B 0

:updateJRE
	rem * get default path from dir like C:\Program Files\IBM\Director\lwi\conf\exescripts\prelaunch_once\
	set "scriptDir=%~dp0"
	set "rootPath=%scriptDir:~0,-36%"
	if not '%1'=='' (
		set "rootPath=%~1"
	)
	if not exist "%rootPath%" (
		echo The root path of Director does not exist: "%rootPath%"
		set RETURN_CODE=1
		goto endlocal
	)
	
	set "logdir="%rootPath%\log""
	if not exist %logdir% (
		mkdir %logdir%
		echo Created log folder: %logdir%
	)
	set logFile="%rootPath%\log\updateinstaller.log"
	echo The log file is %logFile%                                             >> %logFile%
	echo Updating JRE by prelaunch_once at %date% %time%  	                   >> %logFile%
	
	set "jrePath=%rootPath%\jre"
	set "tmpJre=%rootPath%\jre.new"
	set "stamp=%timestamp%"
	REM set "oldJre=%rootPath%\jre.old.%stamp%"
	set "oldJre=%rootPath%\jre.old"
	
	if not exist "%tmpJre%" (
		echo The new JRE is not ready in "%tmpJre%"                            >> %logFile%
		set RETURN_CODE=1
		goto endlocal
	)
	
	call :killOtherProcesses
		
	move /Y "%jrePath%" "%oldJre%"                                             >> %logFile% 
	set RETURN_CODE=%ERRORLEVEL%
	if "%RETURN_CODE%" == "0" (
		echo Moved current JRE to "%oldJre%"                                   >> %logFile%
	) else (
		echo Failed to move current JRE to "%oldJre%"                          >> %logFile%
	)
	
	if exist "%jrePath%" (
		echo Trying to delete JRE "%jrePath%"                                  >> %logFile%
		rmdir /S /Q "%jrePath%"                                                >> %logFile%
		if not exist "%jrePath%" (
			echo Deleted JRE "%jrePath%" successfully.                         >> %logFile%
		) else (
			echo Deletion of JRE "%jrePath%" failed, files or directories may remain. >> %logFile%
			goto endlocal
		)
	)
	
	move /Y "%tmpJre%" "%jrePath%"                                             >> %logFile%
	set RETURN_CODE=%ERRORLEVEL%
	if "%RETURN_CODE%" == "0" (
		echo Moved new JRE from "%tmpJre%" to "%jrePath%" successfully.        >> %logFile%
	) else (
		echo Failed to move new JRE from "%tmpJre%" to "%jrePath%"             >> %logFile%
		goto endlocal
	)
	
	if exist "%oldJre%" (
		rmdir /S /Q "%oldJre%"                                                 >> %logFile%
		if exist "%oldJre%" (
			echo Failed to delete old JRE "%oldJre%" at this time.             >> %logFile%
		) else (
			echo Deleted old JRE "%oldJre%" successfully.                      >> %logFile%
		)
	)
	
	echo Completed JRE update:                                                 >> %logFile%
	"%jrePath%\bin\java" -version                                             2>> %logFile%
	goto endlocal
	
	
:killOtherProcesses
	echo Finding all processes running director JRE...                         >> %logFile%
	wmic process where "CommandLine like '%%Director\\jre\\bin\\%%'"           >> %logFile%
	echo ------                                                                >> %logFile%
	echo Killing processes running director JRE...                             >> %logFile%
	wmic process where "CommandLine like '%%Director\\jre\\bin\\%%'" delete    >> %logFile%
	goto :EOF

:endlocal
	if not %logFile% == "" (
		if not exist "%tmpJre%" (
			del /F /Q "%rootPath%\lwi\conf\exescripts\prelaunch_always\updateServerJRE_always.bat" >> %logFile%
			echo The update ended at %date% %time%                             >> %logFile%
		) else (
			echo The update by prelaunch_once ended at %date% %time%, will try to change java home by prelaunch_always. >> %logFile%
		)
	)
	endlocal
	exit /B 0